<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Item;
use App\Models\ItemImages;
class ItemsController extends MasterController
{
    public function __construct(Item $model)
    {
        $this->model = $model;
        $this->route = 'items';
        $this->module_name = 'المنتجات';
        $this->language = [' بالعربية' => 'ar', ' بالانجليزية' => 'en'];
        $this->json_fields        = ['الإسم'=>'name','التفاصيل'=>'detail'];
        $this->index_fields = ['السعر'=>'price','الكميه'=>'count'];
        $this->create_fields = ['السعر'=>'price','التخفيض'=>'discount','الرمز'=>'symbol','الكميه'=>'count'];


        parent::__construct();

    }
    public function store(Request $request){
         $this->validate($request,[
            'name_en' => 'required','name_ar'=>'required','detail_ar','detail_en'
            ]);
        $item = new Item();
        $item['name->en']  = $request['name_en'];
        $item['name->ar'] = $request['name_ar'];
        $item['detail->ar'] = $request['detail_ar'];
        $item['detail->en'] = $request['detail_en'];
        $item->category_id = $request['category_id'];
        $item->price = $request['price'];
        $item->discount = $request['discount'];
        $item->symbol = $request['symbol'];
        $item->size = $request['size'];
        $item->save();
        $image_count = count($request['images']);
        for ($i = 0; $i < $image_count; $i++){
            $new_image = new ItemImages();
            $new_image->item_id =  $item->id;
            $new_image->color_id = $request['color'][$i];

            if ($request['images'][$i]){
                // $destinationPath = $path . 'app/public/products/'; // upload path
                $extension = $request['images'][$i]->getClientOriginalExtension(); // getting image extension
                $name = time() . '' . rand(11111, 99999) . '.' . $extension; // renameing image
                $request['images'][$i]->move(public_path('images/items'), $name); // uploading file to given path
                $new_image->image=$name;

            }
            $new_image->save();
        }
        return redirect('admin/' . $this->route . '')->with('created', 'تمت الاضافة بنجاح');

    }
    public function update($id,Request $request){

        $item = Item::find($id);
        
        $item['name->en']  = $request['name_en'];
        $item['name->ar'] = $request['name_ar'];
        $item['detail->ar'] = $request['detail_ar'];
        $item['detail->en'] = $request['detail_en'];
        $item->category_id = $request['category_id'];
        $item->price = $request['price'];
        $item->discount = $request['discount'];
        $item->symbol = $request['symbol'];
        $item->size = $request['size'];
        $item->update();
        $image_count = count($request['images']);
        for ($i = 0; $i < $image_count; $i++){
            $new_image = new ItemImages();
            $new_image->item_id =  $item->id;
            $new_image->color_id = $request['color'][$i];

            if ($request['images'][$i]){
                // $destinationPath = $path . 'app/public/products/'; // upload path
                $extension = $request['images'][$i]->getClientOriginalExtension(); // getting image extension
                $name = time() . '' . rand(11111, 99999) . '.' . $extension; // renameing image
                $request['images'][$i]->move(public_path('images/items'), $name); // uploading file to given path
                $new_image->image=$name;

            }
            $new_image->save();
        }
        return redirect('admin/' . $this->route . '')->with('created', 'تمت الاضافة بنجاح');

    }
    public function delete_image($id){
        ItemImages::destroy($id);
        return back()->with('success','تم مسح الصوره');
    }
    public function validation_func($method,$id=null)
    {
        if($method == 1) // POST Case
            return ['name_en' => 'required','name_ar'=>'required','detail_en'=>'required','detail_ar'=>'required','image' => 'required|mimes:png,jpg,jpeg','price'=>'required|numeric|gt:discount','count'=>'required|numeric'];
        return ['name_en' => 'required','name_ar'=>'required','detail_en'=>'required','detail_ar'=>'required','image' => 'required|mimes:png,jpg,jpeg','price'=>'required|numeric|gt:discount','count'=>'required|numeric'];
    }
}
