<?php

namespace App\Http\Controllers\Admin;

use App\Http\Requests;
use App\Http\Controllers\Controller;
use App\Models\AdminRole;
use App\Models\Order;
use App\Models\City;
use App\Models\Product;
use App\Models\Permission;
use App\Models\Role;
use App\Models\RolePermission;
use App\Models\Setting;
use App\Models\Expenses;
use App\Models\Admin;
use App\User;
use function Couchbase\defaultDecoder;
use Illuminate\Http\Request;
use Auth;
use App\Models\Category;
use App\Models\Properties;
abstract class MasterController extends Controller
{

    protected $model;         // model name
    protected $language;         // number of languages
    protected $route;         // route name
    protected $perPage = 10;  // pagination
    protected $validation_c;  // validation on create
    protected $validation_u;  // validation on update
    protected $module_name;
    protected $single_module_name;
    protected $index_fields;
    protected $create_fields;
    protected $update_fields;
    protected $json_fields;
    protected $index_fields_rejected;
    protected $index_fields_waiting;
    protected $index_fields_in_progress;
    protected $index_fields_delivered;
    protected $index_fields_transfered;

    public function __construct()
    {

        $user_array = [];
        $users = \App\User::where(['status' => 'active'])->get();
        foreach ($users as $user) {
            $user_array[$user->id] = $user->name;
        }

        $city_array = [];
        $cities = \App\Models\City::all();
        foreach ($cities as $city) {
            $city_array[$city->id] = $city->name['ar'] . ' - ' . $city->name['en'];
        }
        $category_array = [];
        $categories = \App\Models\Category::all();
        foreach ($categories as $category) {
            $category_array[$category->id] = $category->name['ar'] . ' - ' . $category->name['en'];
        }
        $product_array = [];
        $products = \App\Models\Product::all();
        foreach ($products as $product) {
            $product_array[$product->id] = $product->name['ar'] . ' - ' . $product->name['en'];
        }

        $expenses_array = [];
        $expensess = \App\Models\Expenses::all();
        foreach ($expensess as $expenses) {
            $expenses_array[$expenses->id] = $expenses->name;
        }


        $this->middleware('auth:admin');
        view()->share(array('module_name' => $this->module_name,
            'single_module_name' => $this->single_module_name,
            'route' => $this->route,
            'language' => $this->language,
            'index_fields' => $this->index_fields,
            'index_fields_rejected' => $this->index_fields_rejected,
            'index_fields_waiting' => $this->index_fields_waiting,
            'index_fields_in_progress' => $this->index_fields_in_progress,
            'index_fields_delivered' => $this->index_fields_delivered,
            'index_fields_transfered' => $this->index_fields_transfered,
            'create_fields' => $this->create_fields,
            'update_fields' => $this->update_fields,
            'json_fields' => $this->json_fields,
            'user' => $user_array,
            'category' => $category_array,
            'product' => $product_array,
            'expenses' => $expenses_array,
            'city' => $city_array,
            'status' => ['active' => 'مفعل', 'not_active' => 'غير مفعل'],
            'admin_permission' => ['active' => 'مفعل', 'not_active' => 'غير مفعل'],
            'setting' => Setting::first(),
            'product_count' => Product::count(),
            'city_count' => City::count(),
            'admin_count' => Admin::count(),
            'user_count' => User::where('type', 'driver')->count(),
            'categories_count' => Category::count(),

            'users' => User::all(),

        ));
    }

    public function index()
    {

        $rows = $this->model->latest()->get();
        return view('admin.' . $this->route . '.index', compact('rows'));
    }

    public function create()
    {
        $roles = Role::all();
        $permissions = Permission::all();
        $properties = Properties::where('parent_id',null)->get();
        return view('admin.' . $this->route . '.create', compact('roles', 'permissions','properties'));
    }

    public function store(Request $request)
    {
        if($request->permission){
            if(empty($request->name) || $request->name == '' || $request->name == null){
                return back()->with('error', 'يجب ادخال الاسم');
            }
        }

        $this->validate($request, $this->validation_func(1));
        $obj = $this->model->create($request->all());
        if ($request->role_id) {
            $role = new AdminRole();
            $role->admin_id = $obj->id;
            $role->role_id = $request->role_id;
            $role->save();
        }
        if ($request->permission) {
            foreach ($request->permission as $value) {
                $permission = new RolePermission();
                $permission->permission_id = $value;
                $permission->role_id = $obj->id;
                $permission->save();
            }
        }


        if ($request->images) {
            $i = 0;
            foreach ($request->images as $image) {
                $file = $image;
                $destinationPath = 'images/' . $this->route . '/';
                $filename = $file->getClientOriginalName();
                $file->move($destinationPath, $filename);
                $img[$i] = $filename;
                $i++;
            }
            $obj->images = $img;
            $obj->update();
        }
        if ($this->route == 'admin') {
            return redirect('admin/admins')->with('created', 'تمت الاضافة بنجاح');

        } else {
            return redirect('admin/' . $this->route . '')->with('created', 'تمت الاضافة بنجاح');

        }
    }

    public function edit($id)
    {
        $row = $this->model->find($id);
        $permissions = Permission::all();
        if ($row->permissions) {
            $permissions = $row->permissions;
        }
        $roles = Role::all();
        $properties = Properties::where('parent_id',null)->get();

        return View('admin.' . $this->route . '.edit', compact('row', 'roles', 'permissions','properties'));
    }

    public function update($id, Request $request)
    {
        $this->validate($request, $this->validation_func(2, $id));
        $obj = $this->model->find($id);
        $obj->update($request->all());

        if ($request->role) {
            if (AdminRole::where('admin_id', $id)->first()) {
                AdminRole::where('admin_id', $id)->update(['role_id' => $request->role]);

            } else {
                AdminRole::create(['admin_id' => $id, 'role_id' => $request['role']]);
            }


        }
        if ($request->permission) {
//            DB::table('permission_role')->where('role_id', $role->id)->delete();
            RolePermission::where('role_id', $obj->id)->delete();
            foreach ($request['permission'] as $key => $value) {
                $obj->attachPermission($value);
            }
        } else {
            RolePermission::where('role_id', $obj->id)->delete();

        }


        if ($request->images) {
            $i = 0;
            foreach ($request->images as $image) {
                $file = $image;
                $destinationPath = 'images/' . $this->route . '/';
                $filename = $file->getClientOriginalName();
                $file->move($destinationPath, $filename);
                $img[$i] = $filename;
                $i++;
            }
            $obj->images = $img;
            $obj->update();
        }
        return redirect('admin/' . $this->route . '')->with('updated', 'تم التعديل بنجاح');
    }

    public function destroy($id)
    {
        $this->model->find($id)->forceDelete();
        return redirect('admin/' . $this->route . '')->with('deleted', 'تم الحذف بنجاح');
    }

    public function show($id)
    {

        $row = $this->model->find($id);
        if ($this->route =='orders'){
            $row->seen = 'true';
            $row->update();
        }
        return View('admin.' . $this->route . '.show', compact('row'));
    }

    public function Activate($id)
    {

        $row = $this->model->find($id);
        if ($row->status == 'not_active') {
            $this->model->find($id)->update(['status' => 'active']);
            return back()->with('success', 'تم الغاء التفعيل بنجاح');
        } else {
            $this->model->find($id)->update(['status' => 'not_active']);
            return back()->with('success', 'تم التفعيل بنجاح');
        }
    }
}

