<?php

namespace App\Http\Controllers\Admin;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\Image;
use App\Models\ProductProperties;

class ProductsController extends MasterController
{
    public function __construct(Product $model)
    {
        $this->model = $model;
        $this->route = 'products';
        $this->module_name = 'المنتجات';
        $this->language = [' بالعربية' => 'ar', ' بالانجليزية' => 'en'];
        $this->json_fields = ['الإسم' => 'name', 'الوصف' => 'notes', 'اسم الخاصيه' => 'size_name'];

        parent::__construct();

    }

    public function store(Request $request)
    {
        $this->validate($request, $this->validation_func(1));

        $product = new Product();
        $product['name->en'] = $request['name_en'];
        $product['name->ar'] = $request['name_ar'];
        $product['notes->en'] = $request['notes_en'];
        $product['notes->ar'] = $request['notes_ar'];
        $product['price'] = $request['price'];
        $product['type'] = $request['type'];
        $product['category_id'] = $request['category_id'];
        $product['size_name->en'] = $request['size_name_en'];
        $product['size_name->ar'] = $request['size_name_ar'];
        $product['status'] = $request['status'];
        $product['amount'] = $request['amount'];
        $product['amount_id'] = $request['amount_id'];

        $product->save();

        if ($request->hasFile('image')) {
            foreach ($request['image'] as $image) {
                $images = new Image();
                $images->product_id = $product->id;
                $file_name = str_random(16) . '.' . $image->getClientOriginalExtension();
                $image->move(public_path('images/products'), $file_name);
                $images->image = $file_name;
                $images->save();
            }
        }
        if ($request->properties){
            foreach ($request['properties'] as $property) {
                \DB::table('product_properties')->insert(['product_id'=>$product->id,'property_id'=>$property]);
            }
        }
        return redirect('admin/' . $this->route . '')->with('created', 'تمت الاضافة بنجاح');

    }

    public function update($id, Request $request)
    {
        $this->validate($request, $this->validation_func(2, $id));

        $product = Product::find($id);
        $product['name->en'] = $request['name_en'];
        $product['name->ar'] = $request['name_ar'];
        $product['notes->en'] = $request['notes_en'];
        $product['notes->ar'] = $request['notes_ar'];
        $product['price'] = $request['price'];
        $product['type'] = $request['type'];
        $product['category_id'] = $request['category_id'];
        $product['size_name->en'] = $request['size_name_en'];
        $product['size_name->ar'] = $request['size_name_ar'];
        $product['status'] = $request['status'];
        $product['amount'] = $request['amount'];
        $product['amount_id'] = $request['amount_id'];
        $product->update();

        if ($request->hasFile('image')) {
            foreach ($request['image'] as $image) {
                $images = new Image();
                $images->product_id = $product->id;
                $file_name = str_random(16) . '.' . $image->getClientOriginalExtension();
                $image->move(public_path('images/products'), $file_name);
                $images->image = $file_name;
                $images->save();
            }
        }

        \DB::table('product_properties')->where('product_id','=',$product->id)->delete();
        if ($request['properties']){
            foreach ($request['properties'] as $property) {
                \DB::table('product_properties')->insert(['product_id'=>$product->id,'property_id'=>$property]);
            }

        }

        return redirect('admin/' . $this->route . '')->with('created', 'تمت الاضافة بنجاح');

    }

    public function delete_image($id)
    {

        Image::find($id)->forceDelete();

        return redirect()->back();
    }

    public function validation_func($method, $id = null)
    {
        if ($method == 1) // POST Case
            return ['name_en' => 'required', 'name_ar' => 'required', 'notes_en' => 'required', 'notes_ar' => 'required', 'category_id' => 'required', 'type' => 'required', 'price' => 'required', 'size_name_en' => 'required', 'size_name_ar' => 'required', 'image' => 'required'];
        return ['name_en' => 'required', 'name_ar' => 'required', 'notes_en' => 'required', 'notes_ar' => 'required', 'category_id' => 'required', 'type' => 'required', 'price' => 'required', 'size_name_en' => 'required', 'size_name_ar' => 'required'];
    }

}