<?php

namespace App\Http\Controllers\Admin;

use App\Models\Invoice;
use App\Models\Product;
use App\Models\Store;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

class StoresController extends MasterController
{
    public function __construct(Store $model)
    {
        $this->model = $model;
        $this->route = 'stores';
        $this->module_name = 'المخزن';
        $this->index_fields = ['الاسم' => 'name', 'الكمية بالعدد' => 'count'];
        $this->create_fields = ['الاسم' => 'name', 'الكمية بالعدد' => 'count'];


        parent::__construct();

    }

    public function increase_store(Request $request)
    {

        $this->validate($request, [
            'count' => 'required',
            'row_id'=>'required'
        ]);

        $store = Store::find($request['row_id']);

        $store->count += $request['count'];
        $store->update();
        Invoice::create([
            'title'=>$store->name,
            'type'=>'إضافة',
            'date'=>Carbon::now(),
            'from_table'=>'store',
            'amount'=>$request['count']
        ]);

        return redirect()->back()->with('updated','تم التعديل بنجاح');
    }

    public function decrease_store(Request $request)
    {
        $this->validate($request, [
            'count' => 'required',
            'row_id' => 'required'
        ]);

        $store = Store::find($request['row_id']);
        if ($store->count >= $request['count']){
            $store->count -= $request['count'];
            $store->update();
            Invoice::create([
                'title'=>$store->name,
                'type'=>'سحب',
                'date'=>Carbon::now(),
                'from_table'=>'store',
                'amount'=>$request['count']
            ]);
            return redirect()->back()->with('updated', 'تم التعديل بنجاح');
        }else{
            return redirect()->route('stores.index')->with('error','عفوا الكمية التي تريد سحبها أكثر من الموجود');
        }



    }

    public function increase_product(Request $request)
    {

        $this->validate($request, [
            'count' => 'required',
            'row_id' => 'required'
        ]);

        $store = Product::find($request['row_id']);

        $store->amount += $request['count'];
        $store->update();
        Invoice::create([
            'title' => $store->name['ar'],
            'type' => 'إضافة',
            'date' => Carbon::now(),
            'from_table' => 'product',
            'amount' => $request['count']
        ]);

        return redirect()->back()->with('updated', 'تم التعديل بنجاح');
    }

    public function decrease_product(Request $request)
    {
        $this->validate($request, [
            'count' => 'required',
            'row_id' => 'required'
        ]);

        $store = Product::find($request['row_id']);
        if ($store->amount >= $request['count']) {
            $store->amount -= $request['count'];
            $store->update();
            Invoice::create([
                'title' => $store->name['ar'],
                'type' => 'سحب',
                'date' => Carbon::now(),
                'from_table' => 'product',
                'amount' => $request['count']
            ]);
            return redirect()->back()->with('updated', 'تم التعديل بنجاح');
        } else {
            return redirect()->route('products.index')->with('error', 'عفوا الكمية التي تريد سحبها أكثر من الموجود');
        }


    }

    public function validation_func($method, $id = null)
    {
        if ($method == 1) // POST Case
            return ['name' => 'required', 'count' => 'required'];
        return ['name' => 'required', 'count' => 'required'];
    }
}
