<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Validator;
use App\User;
use abdullahobaid\mobilywslaraval\Mobily;
use Auth;

class MasterController extends Controller
{
    /**
     *basic requirements
     * all active data like users or categories with its relations
     * specific shop or user with relations
     * search
     * delete item
     * add item
     * edit item
     * register,login,update_profile and forget password
     */

    protected $model;
    protected $related_model;
    protected $related_model_name;
    protected $route;
    protected $perPage = 10;
    protected $index_fields;
    protected $apiToken;
    protected $create_fields;
    protected $json_fields;

    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->auth = Auth::guard('api')->user();
            return $next($request);
        });
    }

    public function update_tokens($user, $serial_num, $device_token)
    {
        if ($user->device_token != null) {
            foreach ($user->device_token as $key => $value) {
                if ($serial_num == $key) {
                    $in[$key] = $device_token;
                } else {
                    $in[$key] = $value;
                }
            }
        }
        $in[$serial_num] = $device_token;
        $user->update(['device_token' => $in]);
    }

    public function update_apiToken($user)
    {
        $user->update(['apiToken' => str_random(60)]);
    }

    public function check_apiToken($apiToken)
    {
        if ($apiToken) {
            try {
                $split = explode("sa3d01", $apiToken);
                $user = User::where('apiToken', $split['1'])->first();
            } catch (\Exception $e) {
                return response()->json(['value' => false, 'message' => 'Invalid authentication token in request'], 401);
            }
            if (!$user) {
                return response()->json(['value' => false, 'message' => 'المستخدم غير موجود']);
            }
        } else {
            return response()->json(['value' => false, 'message' => 'Invalid authentication token in request '], 401);
        }
    }

    public function index(Request $request)
    {
        //auth
        $check_token = $this->check_apiToken($request->header('apiToken'));
        if ($check_token && $this->apiToken == true) {
            return $check_token;
        }
        //auth
        $page = $request->header('page', 1);
        $count = $this->model->count();
        $perPage = 10;
        $offset = ($page * $perPage) - $perPage;
        $rows = $this->model->latest()->offset($offset)->limit(10)->get();
        $data = [];
        foreach ($rows as $row) {
            $arr = $row->static_model();
            if ($this->related_model) {
                $related_model_name = $this->related_model_name;
                $arx = [];
                foreach ($row->$related_model_name as $related_row) {
                    $arx[] = $related_row->static_model();
                }
                $arr[$this->related_model_name] = $arx;
            }
            $data[] = $arr;
        }
        if ($count > 0) {
            $pages_count = (integer)round($count / 10);
            if ($pages_count == 0) {
                $pages_count = 1;
            }
        } else {
            $pages_count = 0;
        }
        return response()->json(['value' => true, 'data_count' => $count, 'pages_count' => $pages_count, 'data' => $data]);
    }

    public function store(Request $request)
    {
        //auth
        $check_token = $this->check_apiToken($request->header('apiToken'));
        if ($check_token && $this->apiToken == true && $this->route != 'users') {
            return $check_token;
        } elseif (!$check_token && $this->apiToken == true) {
            $split = explode("sa3d01", $request->header('apiToken'));
            $user = User::where('apiToken', $split['1'])->first();
            $this->update_apiToken(User::find($user->id));
        }
        //auth
        $lang = $request->header('lang', 'ar');
        $validator = Validator::make($request->all(), $this->validation_rules(1), $this->validation_messages($lang));
        if ($validator->passes()) {
            if ($this->route == 'user'){
                $obj = $this->model->create([
                    'username' => $request['username'] ,
                    'password'=>bcrypt($request['password']),
                    'email' =>$request['email'],
                    'mobile'=>$request['mobile'],
                    'apiToken'=>$request['apiToken']
                ]);

            }else{
                $obj = $this->model->create($request->all());

            }
            if ($this->route == 'users') {
                $activation_code = str_random(5);
                //send_sms
                $this->update_tokens(User::find($obj->id), $request->serial_num, $request->device_token);
                $obj->refresh();
                return response()->json(['value' => true, 'activation_code' => $activation_code, 'data' => $obj->static_model()]);
            } elseif ($this->route == 'orders') {
                $user->update(['lat' => $request['client_lat'], 'long' => $request['client_long']]);
                $request['shop_id'] ? $id = $request['shop_id'] : $id = $request['target_id'];
                $obj->notify_drivers($user, $id);
            }
            $obj->refresh();
            if ($this->route == 'user') {
                $code = rand(111, 222);
                $obj->code = $code;
                $obj->update();
                $message = 'Dear Customer your verified code is ' . $code;
                Mobily::send($obj->mobile, $message);
                return response()->json(['value' => true, 'data' => $obj->static_model(),'code'=>(string)$code]);
            }
            return response()->json(['value' => true, 'data' => $obj->static_model()]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function update($id, Request $request)
    {
        //auth
        $check_token = $this->check_apiToken($request->header('apiToken'));
        if ($check_token && $this->apiToken == true) {
            return $check_token;
        }
        //auth
        $lang = $request->header('lang', 'ar');
        $validator = Validator::make($request->all(), $this->validation_rules(2, $id), $this->validation_messages($lang));
        if ($validator->passes()) {
            $row = $this->model->find($id);
            if (!$row) {
                return response()->json(['value' => false]);
            }
            if ($this->route == 'users') {
                $row->update(['username' => $request['username'], 'mobile' => $request['mobile'], 'city' => $request['city'], 'email' => $request['email']]);
                $this->update_tokens($row, $request->serial_num, $request->device_token);
            } else {
                $row->update($request->all());
            }
            $row->refresh();
            return response()->json(['value' => true, 'data' => $row->static_model()]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function show($id, Request $request)
    {
        //auth
        $check_token = $this->check_apiToken($request->header('apiToken'));
        if ($check_token && $this->apiToken == true) {
            return $check_token;
        }
        //auth
        $row = $this->model->find($id);
        if (!$row) {
            return response()->json(['value' => false]);
        }
        return response()->json(['value' => true, 'data' => $row->static_model()]);
    }

    public function destroy($id, Request $request)
    {
        //auth
        $check_token = $this->check_apiToken($request->header('apiToken'));
        if ($check_token && $this->apiToken == true) {
            return $check_token;
        }
        //auth
        $this->model->find($id)->delete();
        return response()->json(['value' => true]);
    }
}