<?php

namespace App\Http\Controllers\Api;

use App\Models\Cart;
use App\Models\Order;
use App\Models\Rate;
use App\User;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Support\Facades\Lang;
use function Couchbase\defaultDecoder;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Validator;
use App\Models\OrderCart;
use App\Models\Setting;
use JWTAuth;
use Carbon\Carbon;


class OrderController extends Controller
{
    public function update_tokens($user, $serial_num, $device_token)
    {
        $user->device_token = null;
        $user->device_token = $device_token;
        $user->update();

    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $lang = $request->header('Accept-Language', 'en');
        \App::setLocale($lang);
        $validator = Validator::make($request->all(), [
            'farouz' => 'required',
            'carts' => 'required_if:farouz,android',
            'ios_carts' => 'required_if:farouz,ios',
            'city_id' => 'required',
            'payment_id' => 'required',
            'mobile' => 'required',
            'client_name' => 'required',
            'image' => 'required_if:payment_id,1',
            'lat' => 'required',
            'lng' => 'required',
        ]);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }


        if ($user) {

            $setting = Setting::first();
            $point1_lat = $setting->location['lat'];
            $point1_long = $setting->location['lng'];
            $point2_lat = $request['lat'];
            $point2_long = $request['lng'];

            $degrees = rad2deg(acos((sin(deg2rad($point1_lat)) * sin(deg2rad($point2_lat))) + (cos(deg2rad($point1_lat)) * cos(deg2rad($point2_lat)) * cos(deg2rad($point1_long - $point2_long)))));
            $distance = $degrees * 111.13384;
            // if ($distance > 30) {
            //     if ($lang == 'en') {
            //         return response()->json(['value' => false, 'msg' => 'Sorry the distance is larger than 30 KM']);

            //     } else {
            //         return response()->json(['value' => false, 'msg' => 'عذرا المسافه تتعدي 30 كيلو متر']);
            //     }
            // }

        }


        if ($validator->passes()) {
            if ($user) {
                if ($request->hasFile('image')) {
                    $filename = md5($request['image']->getClientOriginalName()) . '.' . $request['image']->getClientOriginalExtension();
                    $request['image']->move(public_path('images/orders'), $filename);
                } else {
                    $filename = null;
                }


                $order = Order::create([
                    'user_id' => $user->id,
                    'city_id' => $request['city_id'],
                    'status' => 'waiting',
                    'payment_id' => $request['payment_id'],
                    'mobile' => $request['mobile'],
                    'client_name' => $request['client_name'],
                    'bank_id' => $request['bank_id'],
                    'image' => $filename,
                    'lng' => $request['lng'],
                    'lat' => $request['lat'],
                    'price' => $request['price'],
                    'notes' => $request['notes'],
                    'address'=>$request['address'],

//                    'driver_id' => User::where('type', 'driver')->first()->id
                ]);
                if ($request['carts'] || $request['ios_carts']) {
                    if ($request['farouz'] == 'android') {
                        $array = $request['carts'];
                        foreach (explode(',', $array) as $value) {
                            OrderCart::create([
                                'cart_id' => $value,
                                'order_id' => $order->id
                            ]);
                            Cart::where('id', $value)->update([
                                'status' => 'order'
                            ]);
                        }
                    } elseif ($request['farouz'] == 'ios') {
                        $array_ios = $request['ios_carts'];
                        foreach (json_decode($array_ios) as $item) {
                            OrderCart::create([
                                'cart_id' => $item,
                                'order_id' => $order->id
                            ]);
                            Cart::where('id', $item)->update([
                                'status' => 'order'
                            ]);
                        }
                    }

                }

                User::where('id', $user->id)->update([
//                    'name' => $request['client_name'],
                    'city_id' => $request['city_id'],
//                    'phone' => $request['mobile'],
                    'lng' => $request['lng'],
                    'lat' => $request['lat'],
                ]);


                $data['id'] = $order->id;
                $data['client_name'] = $order->client_name;
                $data['address'] = $order->address ? $order->address : '';
                $data['mobile'] = $order->mobile;
                $data['status'] = $order->status;
                $data['lat'] = $order->lat;
                $data['lng'] = $order->lng;
                $data['payment'] = $order->payment->name[$lang];
                $data['total_price'] = $order->price;
                $data['notes'] = $order->notes ? $order->notes : '';
                $data['time'] = Carbon::parse($order->created_at)->toTimeString();
                $data['date'] = Carbon::parse($order->created_at)->toDateString();
                $data['carts'] = [];
                foreach ($order->order_carts as $cart) {


                    $arr['id'] = $cart->id;
                    $arr['name'] = $cart->product->name[$lang];
                    $arr['type'] = $cart->product->category->name[$lang];
                    $arr['amount'] = (integer)$cart->amount;
                    $arr['price'] = $cart->price * $cart->amount;
                    $arr['amount_type'] = $cart->product->amountType->name[$lang];
                    $arr['properties'] = [];
                    foreach ($cart->properties as $property) {
                        $d['id'] = $property->id;
                        $d['name'] = $property->name[$lang];
                        $d['price'] = $property->price ? $property->price : '';
                        $arr['properties'][] = $d;
                    }
                    $data['carts'][] = $arr;
                }


                if ($lang == 'en') {
                    return response()->json(['value' => true, 'data' => 'Order Done , wait for call', 'msg' => $order->id, 'farouz' => $data]);
                } else {
                    return response()->json(['value' => true, 'data' => 'تم تنفيذ الطلب وسوف يتم الاتصال بك', 'msg' => $order->id,'farouz'=>$data]);

                }
            } else {
                if ($lang == 'en') {
                    return response()->json(['value' => false, 'msg' => 'please enter valid serial number']);
                } else {
                    return response()->json(['value' => false, 'msg' => 'برجاء ادخال سريال صالح']);

                }
            }
        } else {


            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function complete_order($id, Request $request)
    {

        $lang = $request->header('Accept-Language', 'en');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }


        $order = Order::find($id);
        
        $drivder =  User::where('type', 'driver')->first();
        if($drivder){
            $order['driver_id'] = $drivder->id;
            $order['status'] = 'pending';
            $order->update();
        }
        
        $user = User::where('type', 'driver')->first();
        if ($user && $user->device_token) {
            $push = new PushNotification('fcm');
            if ($user->device_type == 'android') {
                $msg = [
                    'data' => [
                        'title' => 'لديك طلب جديد',
                        'sound' => 'default',
                        'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        'body' => 'لديك طلب جديد',

                    ],
                    'priority' => 'high',
                ];
            } else {
                $msg = [
                    'data' => [
                        'title' => 'لديك طلب جديد',
                        'sound' => 'default',
                        'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        'body' => 'لديك طلب جديد',

                    ],
                    'notification' => [
                        'title' => 'لديك طلب جديد',
                        'body' => 'لديك طلب جديد',
                        'sound' => 'default',
                        'click_action' => 'FCM_PLUGIN_ACTIVITY',

                    ],
                    'priority' => 'high',
                ];
            }



            $push->setMessage($msg)
                ->setDevicesToken($user->device_token)
                ->send();
        }

//        dd($push->getFeedback());/

        $data['id'] = $order->id;
        $data['client_name'] = $order->client_name;
        $data['address'] = $order->address;
        $data['mobile'] = $order->mobile;
        $data['status'] = $order->status;
        $data['lat'] = $order->lat;
        $data['lng'] = $order->lng;
        $data['payment'] = $order->payment->name[$lang];
        $data['total_price'] = $order->price;
        $data['notes'] = $order->notes ? $order->notes : '';
        $data['time'] = Carbon::parse($order->created_at)->toTimeString();
        $data['date'] = Carbon::parse($order->created_at)->toDateString();
        $data['carts'] = [];
        foreach ($order->order_carts as $cart) {


            $arr['id'] = $cart->id;
            $arr['name'] = $cart->product->name[$lang];
            $arr['type'] = $cart->product->category->name[$lang];
            $arr['amount'] = (integer)$cart->amount;
            $arr['price'] = $cart->price * $cart->amount;
            $arr['amount_type'] = $cart->product->amountType->name[$lang];
            $arr['properties'] = [];
            foreach ($cart->properties as $property) {
                $d['id'] = $property->id;
                $d['name'] = $property->name[$lang];
                $d['price'] = $property->price ? $property->price : '';
                $arr['properties'][] = $d;
            }
            $data['carts'][] = $arr;
        }

        return response()->json(['value' => true, 'data' => $data]);


    }

    public function cancel_order($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'en');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }
        Order::destroy($id);
        if ($lang == 'en') {
            return response()->json(['value' => true, 'data' => 'Your order is canceled']);

        } else {
            return response()->json(['value' => true, 'data' => 'تم إلغاء طلبكم بنجاح']);

        }
    }

    public function rate(Request $request)
    {
        $lang = $request->header('Accept-Language', 'en');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }


        $order = Order::find($request['order_id']);

        $validator = Validator::make($request->all(), [
            'order_id' => 'required',
            'comment' => 'required',
            'rate' => 'required'
        ]);
        if ($validator->passes()) {
            if ($order) {
                Rate::create([
                    'order_id' => $order->id,
                    'comment' => $request['comment'],
                    'rate' => $request['rate']
                ]);

                if ($lang == 'en') {
                    return response()->json(['value' => true, 'data' => 'Rated !']);
                } else {
                    return response()->json(['value' => true, 'data' => 'تم التقييم']);
                }
            } else {
                if ($lang == 'en') {
                    return response()->json(['value' => false, 'msg' => 'wrong order number']);
                } else {
                    return response()->json(['value' => false, 'msg' => 'برجاء ادخال رقم طلب صحيح']);
                }
            }
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }

        $order = Order::find($id);
        if ($order) {
            $data['id'] = $order->id;
            $data['client_name'] = $order->client_name;
            $data['address'] = $order->address;
            $data['mobile'] = $order->mobile;
            $data['status'] = $order->status;
            $data['lat'] = $order->lat;
            $data['lng'] = $order->lng;
            $data['payment'] = $order->payment->name[$lang];
            $data['total_price'] = $order->price;
            $data['notes'] = $order->notes ? $order->notes : '';
            $data['time'] = Carbon::parse($order->created_at)->toTimeString();
            $data['date'] = Carbon::parse($order->created_at)->toDateString();
            $data['carts'] = [];
            foreach ($order->order_carts as $cart) {


                $arr['id'] = $cart->id;
                $arr['name'] = $cart->product->name[$lang];
                $arr['type'] = $cart->product->category->name[$lang];
                $arr['amount'] = (integer)$cart->amount;
                $arr['price'] = $cart->product->price;
                $arr['amount_type'] = $cart->product->amountType->name[$lang];
                $arr['properties'] = [];
                foreach ($cart->properties as $property) {
                    $d['id'] = $property->id;
                    $d['name'] = $property->name[$lang];
                    $d['price'] = $property->price ? $property->price : '';
                    $arr['properties'][] = $d;
                }
                $data['carts'][] = $arr;
            }
            return response()->json(['value' => true, 'data' => $data]);
        } else {
            return response()->json(['value' => false, 'msg' => 'please enter the id of order']);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }
}
