<?php

namespace App\Http\Controllers\Api;

use App\Models\Cart;
use App\Models\Order;
use App\Models\Rate;
use App\Models\Shop;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Validator;
use App\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Edujugon\PushNotification\PushNotification;
use App\Http\Controllers\Admin;
use Illuminate\Support\Facades\Hash;
use abdullahobaid\mobilywslaraval\Mobily;
use JWTAuth;
use Tymon\JWTAuth\Exceptions\JWTException;
use App\Http\Resources\UserResource;
use Illuminate\Support\Facades\Mail;
use App\Mail\verifyAccount;


class UserController extends MasterController
{
    public function __construct(User $model)
    {
        $this->model = $model;
        $this->route = 'user';
        $this->lang = request()->header('lang', 'ar');
        $this->api_token = true;
        $this->index_fields = ['id', 'username', 'email', 'mobile', 'image', 'api_token', 'gender', 'country', 'country_code', 'birthday'];
        parent::__construct();
    }


    public function validation_rules($method, $id = null)
    {
        $rules = ['mobile' => 'unique:users|regex:/^[0-9]+$/', 'email' => 'unique:users', 'device_token' => 'required', 'social_token' => 'unique:users', 'serial_num' => 'required', 'username' => 'required'];
        if ($method == 2) {
            $rules['mobile'] = 'required|regex:/^[0-9]+$/|unique:users,mobile,' . $id;
            $rules['email'] = 'required|email|max:255|unique:users,email,' . $id;
            $rules['social_token'] = 'unique:users,social_token,' . $id;
            $rules['username'] = 'unique:users' . $id;
        }
        return $rules;
    }

    public function validation_messages($lang)
    {
        $messsages = [];
        if ($lang == 'ar') {
            $messsages = array('email.unique' => 'هذا البريد مسجل بالفعل', 'mobile.unique' => 'هذا الهاتف مسجل بالفعل', 'username.unique' => 'اسم المستخدم مسجل بالفعل',);
        }
        return $messsages;
    }
    public function logout(Request  $request){
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);

            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
            }
            $user->device_token = null;
            $user->update();

            JWTAuth::invalidate(JWTAuth::getToken());
            return response()->json(['value' => true, 'data' => 'تم تسجيل الخروج', 'code' => 200]);



    }

    public function device_token(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'device_token' => 'required',
            'device_type' => 'required'
        ]);

        if ($validator->passes()) {


            $all_devices = User::where('device_token', '!=', null)->pluck('device_token');


            foreach ($all_devices as $device) {
                if ($device === $request['device_token']) {
                    return response()->json(['value' => true, 'data' => 'token already exists']);
                }
            }

            User::create([
                'device_token' => $request['device_token'],
                'device_type' => $request['device_type']
            ]);


            return response()->json(['value' => true, 'data' => 'token created']);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function confirm_active(Request $request)
    {
        $lang = $request->header('Accept-Language');
        $validator = Validator::make($request->all(), [
            'email' => 'required',
            'code' => 'required'
        ]);
        if ($validator->passes()) {
            $user = User::where('email', $request['email'])->first();
            if ($user) {
                if ($user->code == $request['code']) {
                    $user->status = 'active';
                    $user->update();


                    $token = JWTAuth::fromUser($user);
//        $data = new UserResource($user);
                    $data['id'] = $user->id;
                    $data['username'] = $user->username;
                    $data['email'] = $user->email;
                    if ($user->mobile == null) {
                        $data['mobile'] = ' ';

                    } else {
                        $data['mobile'] = $user->mobile;
                    }
                    if ($user->address == null) {
                        $data['address'] = ' ';

                    } else {
                        $data['address'] = $user->address;
                    }
                    if ($user->lat == null) {
                        $data['lat'] = ' ';

                    } else {
                        $data['lat'] = $user->lat;
                    }
                    if ($user->lng == null) {
                        $data['lng'] = ' ';

                    } else {
                        $data['lng'] = $user->lng;
                    }
                    if ($user->image == null) {
                        $data['image'] = '';
                    } else {
                        $data['image'] = $user->image;
                    }
                    $data['token'] = $token;
                    if ($user->code == null) {
                        $data['code'] = '';
                    } else {
                        $data['code'] = $user->code;
                    }


                    if ($lang == 'en') {
                        return response()->json(['value' => true, 'data' => $data]);
                    } else {
                        return response()->json(['value' => true, 'data' => $data]);

                    }
                } else {
                    if ($lang == 'en') {
                        return response()->json(['value' => false, 'msg' => 'wrong code']);
                    } else {
                        return response()->json(['value' => false, 'msg' => 'لكود خطأ']);
                    }
                }
            } else {
                if ($lang == 'en') {
                    return response()->json(['value' => false, 'msg' => 'there is no user with this email']);
                } else {
                    return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا الايميل']);
                }
            }
        } else {


            foreach ((array)$validator->errors() as $value) {
                if (isset($value['email'])) {
                    if ($lang == 'en') {
                        $msg = ' please enter email';

                    } else {
                        $msg = 'قم بإدخال البريد الإليكتروني ';

                    }
                    return response()->json(['value' => false, 'msg' => $msg]);
                } elseif (isset($value['code'])) {
                    if ($lang == 'en') {
                        $msg = ' please enter code';

                    } else {
                        $msg = 'قم بإدخال الكود';

                    }
                    return response()->json(['value' => false, 'msg' => $msg]);
                } else {
                    if ($lang == 'en') {
                        $msg = ' something wrong';

                    } else {
                        $msg = 'حدث خطا ما';

                    }
                    return response()->json(['value' => false, 'msg' => $msg]);
                }
            }


        }
    }

//    public function login(Request $request)
//    {
//        if (is_numeric($request['mobile'])) {
//            $data = ['mobile' => $request->input('mobile'), 'password' => $request->input('password'), 'status' => 'active'];
//            $false_data = ['mobile' => $request->input('mobile'), 'password' => $request->input('password'), 'status' => 'not_active'];
//            $user = User::where(['mobile' => $request->input('mobile')])->first();
//        } else {
//            $data = ['email' => $request->input('mobile'), 'password' => $request->input('password'), 'status' => 'active'];
//            $false_data = ['email' => $request->input('mobile'), 'password' => $request->input('password'), 'status' => 'not_active'];
//            $user = User::where(['email' => $request->input('mobile')])->first();
//        }
//        $lang = $request->header('lang', 'ar');
//
//
//        if (!$user) {
//            $lang == 'ar' ? $msg = 'هذا المستخدم غير مسجل' : $msg = 'This user is not registered';
//            return response()->json(['value' => false, 'msg' => $msg]);
//        }
//
//        if (Auth::attempt($data)) {
//            $user = Auth::user();
//
//            $this->update_tokens($user, $request->serial_num, $request->device_token);
//            $user->refresh();
//            return response()->json(['value' => true, 'data' => $user->static_model(), 'code' => ""]);
//        } elseif (Auth::attempt($false_data)) {
//            $lang == 'ar' ? $msg = 'يرجى التواصل مع الادارة وبرجاء إدخال الكود' : $msg = 'Please contact the administration and check your mobile code';
//            return response()->json(['value' => true, 'data' => $user->static_model(), 'code' => (string)$user->code]);
//        } else {
//            $lang == 'ar' ? $msg = 'كلمة المرور غير صحيحة' : $msg = 'The password is incorrect';
//            return response()->json(['value' => false, 'msg' => $msg]);
//        }
//    }

    public function verify_code(Request $request)
    {
        $user = User::where('mobile', $request['mobile'])->first();
        $lang = $request->header('Accept-Language');

        if ($user) {
            if ($user->code == $request['code']) {
                $user->code = null;
                $user->status = 'active';
                $user->update();
                if ($lang == 'en') {

                    return response()->json(['value' => true, 'data' => $user->static_model()]);


                } else {
                    return response()->json(['value' => true, 'data' => $user->static_model()]);

                }

            } else {
                if ($lang == 'en') {
                    return response()->json(['value' => false, 'msg' => 'error code']);

                } else {
                    return response()->json(['value' => false, 'msg' => ' الكود خطأ']);

                }
            }
        } else {
            if ($lang == 'ar') {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم برقم الجوال']);

            } else {
                return response()->json(['value' => false, 'msg' => 'there is no user with this mobile']);

            }
        }

    }

    public function forget_password(Request $request)
    {
        $lang = $request->header('Accept-Language');
        $validator = Validator::make($request->all(), ['phone' => 'required']);
        if ($validator->passes()) {
            $user = User::where('phone', $request['phone'])->first();
            if ($user) {
                $user_code = rand(1111, 2222);
                $user->code = $user_code;
                $user->update();
//                Mobily::send($user->mobile, $message);
//                Mail::to($user->email)->send(new verifyAccount($user_code));
                $msg = " عميلنا العزيز كود التفعيل الخاص بكم هو $user->code ";
                shamel_sms($user->phone, $msg);

                if ($lang == 'en') {
                    return response()->json(['value' => true, 'data' => 'code sent please check your messages', 'code' => $user_code]);

                } else {
                    return response()->json(['value' => true, 'data' => 'تم إرسال الكود برجاء التحقق من الرسائل', 'code' => $user_code]);

                }
            } else {
                if ($lang == 'en') {
                    return response()->json(['value' => false, 'msg' => 'there is no user with this mobile']);

                } else {
                    return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا الرقم']);

                }

            }
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function check_forget(Request $request)
    {

        $validator = Validator::make($request->all(), ['phone' => 'required', 'code' => 'required']);
        $lang = $request->header('Accept-Language');

        if ($validator->passes()) {
            $user = User::where('phone', $request['phone'])->first();
            if ($user) {
                if ($user->code == $request['code']) {

                    if ($lang == 'en') {
                        return response()->json(['value' => true, 'data' => 'correct code']);

                    } else {
                        return response()->json(['value' => true, 'data' => 'تم إدخال الكود بنجاح']);

                    }
                } else {
                    if ($lang == 'en') {
                        return response()->json(['value' => false, 'msg' => 'code not match']);

                    } else {
                        return response()->json(['value' => false, 'msg' => 'الكود خطأ']);

                    }
                }
            } else {
                if ($lang == 'ar') {
                    return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم برقم الجوال']);

                } else {
                    return response()->json(['value' => false, 'msg' => 'there is no user with this mobile ']);

                }
            }
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function change_password(Request $request)
    {
        $lang = $request->header('Accept-Language');

        $validator = Validator::make($request->all(), ['password' => 'required', 'phone' => 'required']);
        if ($validator->passes()) {
            $user = User::where('phone', $request['phone'])->first();
            if ($user) {
                $user->password = bcrypt($request['password']);
                $user->update();

                if ($request->serial_num && $request->device_token) {
                    $this->update_tokens($user, $request->serial_num, $request->device_token);
                }
                $token = JWTAuth::fromUser($user);
//        $data = new UserResource($user);
                $data['id'] = $user->id;
                $data['username'] = $user->username ? $user->username : "";
                $data['email'] = $user->email ? $user->email : "";
                $data['phone'] = $user->phone ? $user->phone : "";
                $data['address'] = $user->address ? $user->address : "";

                 
                if ($user->lat == null) {
                    $data['lat'] = ' ';

                } else {
                    $data['lat'] = $user->lat;
                }
                if ($user->lng == null) {
                    $data['lng'] = ' ';

                } else {
                    $data['lng'] = $user->lng;
                }
                if ($user->image == null) {
                    $data['image'] = '';
                } else {
                    $data['image'] = $user->image;
                }
                $data['token'] = $token;
                if ($user->code == null) {
                    $data['code'] = '';
                } else {
                    $data['code'] = $user->code;
                }

                if ($lang == 'ar') {
                    return response()->json(['value' => true, 'data' => $data]);

                } else {
                    return response()->json(['value' => true, 'data' => $data]);

                }


            } else {


                if ($lang == 'ar') {
                    return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم برقم الجوال']);

                } else {
                    return response()->json(['value' => false, 'msg' => 'there is no user with this mobile']);

                }


            }
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function update_profile(Request $request)
    {
        $lang = $request->header('Accept-Language');

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }


        $validator = Validator::make($request->all(), ['email' => 'unique:users,email,' . $user->id, 'mobile' => 'unique:users,mobile,' . $user->id]);
        if ($validator->passes()) {

            if ($user) {
                $this->update_tokens($user, $request->serial_num, $request->device_token);
                $user->username = $request['username'];
                $user->email = $request['email'];
                if (!$request->password == null) {
                    $user->password = bcrypt($request['password']);
                }
                $user->address = $request['address'];
                $user->mobile = $request['mobile'];
                $user->prefered_time = $request['prefered_time'];
                $user->lat = $request['lat'];
                $user->lng = $request['lng'];
                if ($request->image) {
                    $filenmae = md5($request['image']->getClientOriginalName()) . '.' . $request['image']->getClientOriginalExtension();
                    $request['image']->move(public_path('images/user'), $filenmae);
                    $user->image = $filenmae;
                }
                $user->update();

                $token = JWTAuth::fromUser($user);

                $data['id'] = $user->id;
                $data['username'] = $user->username;
                $data['email'] = $user->email;
                if ($user->mobile == null) {
                    $data['mobile'] = '';

                } else {
                    $data['mobile'] = $user->phone;
                }
                if ($user->address == null) {
                    $data['address'] = '';

                } else {
                    $data['address'] = $user->address;
                }
                if ($user->lat == null) {
                    $data['lat'] = '';

                } else {
                    $data['lat'] = $user->lat;
                }
                if ($user->lng == null) {
                    $data['lng'] = '';

                } else {
                    $data['lng'] = $user->lng;
                }
                if ($user->image == null) {
                    $data['image'] = '';
                } else {
                    $data['image'] = $user->image;
                }
                $data['token'] = $token;
                if ($user->code == null) {
                    $data['code'] = '';
                } else {
                    $data['code'] = $user->code;
                }


                if ($lang == 'en') {
                    return response()->json(['value' => true, 'data' => $data]);

                } else {
                    return response()->json(['value' => true, 'data' => $data]);

                }
            } else {
                if ($lang == 'en') {
                    return response()->json(['value' => false, 'msg' => 'there is no user']);

                } else {
                    return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم']);

                }
            }
        } else {
            foreach ((array)$validator->errors() as $value) {
                if (isset($value['username'])) {
                    if ($lang == 'en') {
                        $msg = 'this email already taken';

                    } else {
                        $msg = 'هذا الإسم مستخدم من قبل';

                    }

                    return response()->json(['value' => false, 'msg' => $msg]);
                } elseif (isset($value['email'])) {
                    if ($lang == 'en') {
                        $msg = ' this email already taken';

                    } else {
                        $msg = 'هذا البريد الإليكتروني مستخدم من قبل';

                    }
                    return response()->json(['value' => false, 'msg' => $msg]);
                } elseif (isset($value['mobile'])) {
                    if ($lang == 'en') {
                        $msg = ' this mobile already taken';

                    } else {
                        $msg = ' رقم الهاتف مستخدم من قبل';

                    }
                    return response()->json(['value' => false, 'msg' => $msg]);
                } else {
                    if ($lang == 'en') {
                        $msg = ' something wrong';

                    } else {
                        $msg = 'حدث خطا ما';

                    }
                    return response()->json(['value' => false, 'msg' => $msg]);
                }
            }


        }

    }

    public function start_order($id)
    {
        $lang = \request()->header('Accept-Language');

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }

        $order = Order::where('id', $id)->update([

            'trip_start' => Carbon::now(),
        ]);
        return response()->json(['value' => true, 'data' => 'Done !']);
    }

    public function end_order($id)
    {
        $lang = \request()->header('Accept-Language');

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }

        $order = Order::where('id', $id)->update([

            'trip_end' => Carbon::now(),
            'status' => 'finish'
        ]);
        return response()->json(['value' => true, 'data' => 'Done !']);
    }


    // JWT
    public function authenticate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'phone' => 'required',
            'password' => 'required',
            'serial_num' => 'required',
            'device_token' => 'required',
            'device_type' => 'required',
            'type' => 'nullable|in:client,driver'
        ]);
        if ($validator->passes()){
            $lang = $request->header('Accept-Language');

//        dd(User::where('email',$request['email'])->First());


            $auth = ['phone' => $request->input('phone'), 'password' => $request->input('password'), 'status' => 'active','type'=>$request['type']];
//        $token = JWTAuth::attempt($credentials, ['exp' => Carbon\Carbon::now()->addDays(7)->timestamp]);


            try {
                if (!$token = JWTAuth::attempt($auth, ['exp' => Carbon::now()->addDays(7300)->timestamp])) {


                    if ($lang == 'en') {
                        return response()->json(['value' => false, 'msg' => 'Phone or password incorrect'], 200);

                    } else {
                        return response()->json(['value' => false, 'msg' => 'رقم الجوال او الرقم السري غير صحيح'], 200);

                    }
                }
            } catch (JWTException $e) {
                return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
            }


            $user = User::where(['phone' => $request['phone']])->first();
//        $this->update_tokens($user, $request->serial_num, $request->device_token);
            $user->serial_number = $request['serial_num'];
            $user->device_token = $request['device_token'];
            $user->device_type = $request['device_type'] ? $request['device_type'] : 'ios';
            $user->update();
//        if ($user->status == 'not_active'){
//            if ($lang == 'en'){
//                return response()->json(['value'=>false,'msg'=>'please active you account first']);
//
//            }else{
//                return response()->json(['value'=>false,'msg'=>'برجاء قم بتفعيل الحساب أولا']);
//
//            }
//        }

            $data['id'] = $user->id;
            $data['name'] = $user->name;
            $data['phone'] = $user->phone;
            $data['type'] = $user->type;
            $data['token'] = $token;
            return response()->json(['value' => true, 'data' => $data]);
        }else{
            return  response()->json(['value'=>false,'msg'=>$validator->errors()->first()]);
        }


    }

    public function register(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'phone' => 'required|string|max:255|unique:users,phone',
            'name' => 'required',
            'address' => 'nullable',
            'password' => 'required|string|min:6',
            'lat' => 'nullable',
            'lng' => 'nullable',
            'device_token' => 'required',
            'device_type' => 'required|in:android,ios'
        ]);

        if ($validator->fails()) {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }

        $user = User::create(['name' => $request->get('name'), 'phone' => $request->get('phone'), 'password' => bcrypt($request['password']), 'device_type' => $request['device_type'], 'device_token' => $request['device_token'], 'lat' => $request['lat'], 'lng' => $request['lng'], 'address' => $request['address'], 'type' => 'client']);
        $token = JWTAuth::fromUser($user);
//        $data = new UserResource($user);
        $data['id'] = $user->id;
        $data['name'] = $user->name;
        $data['phone'] = $user->phone ? $user->phone : '';
        $data['address'] = $user->address ? $user->address : '';
        $data['lat'] = $user->lat ? $user->lat : '';
        $data['lng'] = $user->lng ? $user->lng : '';
        $data['type'] = 'client';
        $data['token'] = $token;

        return response()->json(['value' => true, 'data' => $data]);
    }

    public function user_login(Request $request)
    {
        $lang = $request->header('Accept-Language');

        $validator = Validator::make($request->all(), [
            'phone' => 'required',
            'password' => 'required',
            'device_token' => 'required',
            'device_type' => 'required'
        ]);
        if ($validator->passes()) {

//        dd(User::where('email',$request['email'])->First());

            $credentials = $request->only('phone', 'password');


//        $token = JWTAuth::attempt($credentials, ['exp' => Carbon\Carbon::now()->addDays(7)->timestamp]);


            try {
                if (!$token = JWTAuth::attempt(['phone' => $request['phone'], 'password' => $request['password'], 'type' => 'client'], ['exp' => Carbon::now()->addDays(7300)->timestamp])) {


                    if ($lang == 'en') {
                        return response()->json(['value' => false, 'msg' => 'Phone or password incorrect'], 200);

                    } else {
                        return response()->json(['value' => false, 'msg' => 'رقم الجوال او الرقم السري غير صحيح'], 200);

                    }
                }
            } catch (JWTException $e) {
                return response()->json(['value' => false, 'msg' => 'could_not_create_token'], 500);
            }


            $user = User::where(['phone' => $request['phone']])->first();
//        $this->update_tokens($user, $request->serial_num, $request->device_token);
            $user->serial_number = $request['serial_num'];
            $user->device_token = $request['device_token'];
            $user->device_type = $request['device_type'] ? $request['device_type'] : 'ios';
            $user->update();
//        if ($user->status == 'not_active'){
//            if ($lang == 'en'){
//                return response()->json(['value'=>false,'msg'=>'please active you account first']);
//
//            }else{
//                return response()->json(['value'=>false,'msg'=>'برجاء قم بتفعيل الحساب أولا']);
//
//            }
//        }

            $data['id'] = $user->id;
            $data['name'] = $user->name;
            $data['phone'] = $user->phone ? $user->phone : '';
            $data['address'] = $user->address ? $user->address : '';
            $data['lat'] = $user->lat ? $user->lat : '';
            $data['lng'] = $user->lng ? $user->lng : '';
            $data['type'] = 'client';
            $data['token'] = $token;
            return response()->json(['value' => true, 'data' => $data]);
        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }

    public function getAuthenticatedUser()
    {
        try {

            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['user_not_found'], 404);
            }

        } catch (Tymon\JWTAuth\Exceptions\TokenExpiredException $e) {

            return response()->json(['token_expired'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\TokenInvalidException $e) {

            return response()->json(['token_invalid'], $e->getStatusCode());

        } catch (Tymon\JWTAuth\Exceptions\JWTException $e) {

            return response()->json(['token_absent'], $e->getStatusCode());

        }

        return response()->json(compact('user'));
    }


    public function driver_orders(Request $request)
    {
        $lang = $request->header('Accept-Language');
        \App::setLocale($lang);

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }
        $orders = Order::where('driver_id', $user->id)->where('status', 'pending')->orderBy('created_at', 'DESC')->get();
        $data = [];
        foreach ($orders as $order) {
            $arr['id'] = $order->id;
            $arr['client_name'] = $order->client_name;
            $arr['mobile'] = $order->mobile;
            // $arr['address'] = $order->address;
            $arr['lat'] = $order->lat;
            $arr['lng'] = $order->lng;
            $arr['time'] = Carbon::parse($order->created_at)->toTimeString();
            $arr['date'] = Carbon::parse($order->created_at)->toDateString();
            if ($order->trip_start == null) {
                $arr['status'] = false;
            } else {
                $arr['status'] = true;
            }
            $data[] = $arr;
        }
        return response()->json(['value' => true, 'data' => $data]);
    }

    public function my_orders($flag, Request $request)
    {
        $lang = $request->header('Accept-Language', 'ar');
        \App::setLocale($lang);

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }
        $orders = Order::where('user_id', $user->id)->where('status', $flag)->orderBy('created_at', 'DESC')->get();
        $data = [];
        foreach ($orders as $order) {

            $arr['id'] = $order->id;
            $arr['client_name'] = $order->client_name;
            // $arr['address'] = $order->address;
            $arr['mobile'] = $order->mobile;
            $arr['lat'] = $order->lat;
            $arr['lng'] = $order->lng;
            $arr['payment'] = $order->payment->name[$lang];
            $arr['total_price'] = $order->price;
            $arr['time'] = Carbon::parse($order->created_at)->toTimeString();
            $arr['date'] = Carbon::parse($order->created_at)->toDateString();
            $arr['carts'] = [];
            foreach ($order->order_carts as $cart) {


                $crt['id'] = $cart->id;
                $crt['name'] = $cart->product->name[$lang];
                $crt['type'] = $cart->product->category->name[$lang];
                $crt['amount'] = (integer)$cart->amount;
                $crt['amount_type'] = $cart->product->amountType->name[$lang];
                $crt['price'] = $cart->price * $cart->amount;
                $crt['properties'] = [];
                foreach ($cart->properties as $property) {
                    $d['id'] = $property->id;
                    $d['name'] = $property->name[$lang];
                    $d['price'] = $property->price ? $property->price : '';
                    $crt['properties'][] = $d;
                }
                $arr['carts'][] = $crt;

            }


            $data[] = $arr;


        }
        return response()->json(['value' => true, 'data' => $data]);
    }

    public function order_search(Request $request)
    {
        $lang = $request->header('Accept-Language', 'en');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id'
        ]);
        if ($validator->passes()) {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                if ($lang == 'en') {
                    return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

                } else {
                    return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

                }
            }


            $order = Order::find($request['order_id']);
            if (!$order) return response()->json(['value' => false, 'msg' => 'لا يوجد طلب بهذا الرقم']);
            $data = [];
            $data['id'] = $order->id;
            $data['client_name'] = $order->client_name;
            // $data['address'] = $order->address;
            $data['mobile'] = $order->mobile;
            $data['lat'] = $order->lat;
            $data['lng'] = $order->lng;
            $data['payment'] = $order->payment->name[$lang];
            $data['total_price'] = $order->price;
            $data['time'] = Carbon::parse($order->created_at)->toTimeString();
            $data['date'] = Carbon::parse($order->created_at)->toDateString();
            $data['carts'] = [];
            foreach ($order->order_carts as $cart) {


                $arr['id'] = $cart->id;
                $arr['name'] = $cart->product->name[$lang];
                $arr['type'] = $cart->product->category->name[$lang];
                $arr['amount'] = (integer)$cart->amount;
                $arr['amount_type'] = $cart->product->amountType->name[$lang];
                $arr['price'] = $cart->price * $cart->amount;
                $arr['properties'] = [];
                foreach ($cart->properties as $property) {
                    $d['id'] = $property->id;
                    $d['name'] = $property->name[$lang];
                    $d['price'] = $property->price ? $property->price : '';
                    $arr['properties'][] = $d;
                }
                $data['carts'][] = $arr;


            }
            return response()->json(['value' => true, 'data' => $data]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }

    public function single_order($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'en');
        \App::setLocale($lang);

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }


        $order = Order::find($id);
        $data = [];
        $data['id'] = $order->id;
        $data['client_name'] = $order->client_name;
        // $data['address'] = $order->address;
        $data['mobile'] = $order->mobile;
        $data['lat'] = $order->lat;
        $data['lng'] = $order->lng;
        $data['payment'] = $order->payment->name[$lang];
        $data['total_price'] = $order->price;
        $data['time'] = Carbon::parse($order->created_at)->toTimeString();
        $data['date'] = Carbon::parse($order->created_at)->toDateString();
        $data['carts'] = [];
        foreach ($order->order_carts as $cart) {


            $arr['id'] = $cart->id;
            $arr['name'] = $cart->product->name[$lang];
            $arr['type'] = $cart->product->category->name[$lang];
            $arr['amount'] = (integer)$cart->amount;
            $arr['amount_type'] = $cart->product->amountType->name[$lang];
            $arr['price'] = $cart->price * $cart->amount;
            $arr['properties'] = [];
            foreach ($cart->properties as $property) {
                $d['id'] = $property->id;
                $d['name'] = $property->name[$lang];
                $d['price'] = $property->price ? $property->price : '';
                $arr['properties'][] = $d;
            }
            $data['carts'][] = $arr;

        }
        return response()->json(['value' => true, 'data' => $data]);
    }

    public function change_order($id, Request $request)
    {
        $lang = $request->header('Accept-Language', 'en');
        \App::setLocale($lang);

        if (!$user = JWTAuth::parseToken()->authenticate()) {
            if ($lang == 'en') {
                return response()->json(['value' => false, 'msg' => 'there is no user with this token']);

            } else {
                return response()->json(['value' => false, 'msg' => 'لا يوجد مستخدم بهذا التوكن']);

            }
        }
        $order = Order::find($id);
        if ($order) {
            Order::where('id', $id)->update([
                'status' => 'finish'
            ]);
            return response()->json(['value' => true, 'data' => 'Done !']);
        } else {
            return response()->json(['value' => false, 'msg' => 'wrong id for order']);
        }


    }
    
    public function appleValidate()
    {
            return response()->json(['value' => true, 'data' => 1]);

    }

}
