<?php

namespace App\Http\Controllers\Auth;

use App\User;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Illuminate\Foundation\Auth\RegistersUsers;

class RegisterController extends Controller
{
    /*
    |--------------------------------------------------------------------------
    | Register Controller
    |--------------------------------------------------------------------------
    |
    | This controller handles the registration of new users as well as their
    | validation and creation. By default this controller uses a trait to
    | provide this functionality without requiring any additional code.
    |
    */

    use RegistersUsers;

    /**
     * Where to redirect users after registration.
     *
     * @var string
     */
    protected $redirectTo = '/';

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('guest');
    }

    /**
     * Get a validator for an incoming registration request.
     *
     * @param  array  $data
     * @return \Illuminate\Contracts\Validation\Validator
     */
    protected function validator(array $data)
    {
        return Validator::make($data, [
            'first_name' => 'required',
            'email' => 'required|email|max:255|unique:users',
            'password' => 'required|string|min:6|confirmed',
            'last_name'=>'required',
            'mobile'=>'required|unique:users'
        ],[
            'first_name.required'=>'يرجي إدخال الاسم الأول',
            'last_name.required'=>'يرجي إدخال الأسم الأخير',
            'password.required'=>'يرجي إدخال الرقم السري',
            'password.min'=>'يرجي إدخال الرقم السري أكبر من 6 حرف',
            'password.confirmed'=>'تأكيد رقم السري غير متطابق',
            'email.required'=>'يرجي إدخال البريد الإلكتروني',
            'email.email'=>'يرجي إدخال البريد الإلكتروني بطريقه صحيحه',
            'email.unique'=>'هذا البريد الإليكتروني مسجل من قبل',
            'mobile.unique'=>'يرجي إدخال رقم الجوال',
        ]);
    }

    /**
     * Create a new user instance after a valid registration.
     *
     * @param  array  $data
     * @return \App\User
     */
    protected function create(array $data)
    {
        return User::create([
            'first_name' => $data['first_name'],
            'email' => $data['email'],
            'password' => bcrypt($data['password']),
            'last_name'=>$data['last_name'],
            'mobile'=>$data['mobile'],
        ]);
    }
}
