<?php

namespace App\Http\Controllers;

use App\Models\Bank;
use App\Models\Category;
use App\Mail\ForgetPassword;
use App\Models\City;
use App\Models\Properties;
use App\Models\CartProperties;
use App\Models\Contact;
use App\Models\Order;
use App\Models\OrderCart;
use App\Models\Payment;
use App\Models\Product;
use App\Models\Rate;
use App\Models\Setting;
use App\Models\Size;
use App\Models\Slider;
use App\User;
use Illuminate\Http\Request;
use Auth;
use LaravelPayfort\Services\Payfort;
use Session;
use Illuminate\Support\Facades\Artisan;
use abdullahobaid\mobilywslaraval\Mobily;
use Gloudemans\Shoppingcart\Facades\Cart;

class HomeController extends Controller
{
    public function __construct()
    {
        view()->share(
            [
                'setting' => Setting::first(),
                'categories' => Category::all(),
                'cities' => City::all(),
                'payments' => Payment::all()
            ]
        );

    }

    public function change_language($lang)
    {
        if ($lang == 'en' || $lang == 'ar') {
            if (!\Session::has('language')) {
                \Session::put('language', $lang);
            } else {
                \Session::put('language', $lang);
            }
            return redirect()->back();
        } else {
            return redirect('/home');
        }
    }

    public function index()
    {
        //return redirect('admin/login');
        
        $sliders = Slider::all();
        return view('web.index', compact('sliders'));
    }

    public function about()
    {
        return view('web.about');
    }

    public function contact()
    {
        return view('web.contact');
    }

    public function policy()
    {
        return view('web.policy');
    }

    public function terms()
    {
        return view('web.terms');
    }

    public function single_category($id)
    {
        $category = Category::find($id);
        return view('web.single_category', compact('category'));
    }

    public function single_product($id)
    {
        $product = Product::find($id);


        $parents = [];
        foreach ($product->properties as $property) {
            $parents[] = $property->parent_id;
        }
        $unique_parents = array_unique($parents);
        $properties = Properties::whereIn('id', $unique_parents)->get();
        return view('web.single_product', compact('product', 'properties'));
    }

//    public function add_to_cart(Request $request)
//    {
//        $this->validate($request, [
//            'amount' => 'required'
//        ]);
//        $product = Product::find($request['product_id']);
//
//        $serial = request()->ip();
//        $client = User::where('serial_number', $serial)->first();
//        if ($client) {
//            $user = $client;
//        } else {
//            $user = User::create(['serial_number' => $serial]);
//        }
//        if ($request['size']) {
//            $size = Size::find($request['size']);
//            Cart::add($product->id, $product->name[\App::getLocale()], 1, $size->price);
//            $cart = \App\Models\Cart::create([
//                'product_id' => $request['product_id'],
//                'size_id' => $request['size_id'],
//                'price' => $request['amount'] * $size->price,
//                'status' => 'cart',
//                'amount' => $request['amount'],
////                'notes' => $request['notes'],
//                'user_id' => $user->id,
//
//            ]);
//        } else {
//            Cart::add($product->id, $product->name[\App::getLocale()], 1, $product->price);
//            $cart = \App\Models\Cart::create([
//                'product_id' => $request['product_id'],
//                'size_id' => $request['size_id'],
//                'amount' => $request['amount'],
//                'price' => $request['amount'] * $product->price,
//                'status' => 'cart',
////                'notes' => $request['notes'],
//                'user_id' => $user->id,
//            ]);
//        }
//
//        if ($request->properties) {
//            foreach ($request->properties as $property) {
//                $cart_properties = new CartProperties();
//                $cart_properties->cart_id = $cart->id;
//                $cart_properties->property_id = $property;
//                $cart_properties->save();
//                $price = Properties::find($property);
//                if ($price) {
//                    $data[] = (integer)$price->price;
//                }
//            }
//            $all_price = array_sum($data) + $cart->price;
//            $cart_update = \App\Models\Cart::find($cart->id);
//            $cart_update->price = $all_price;
//            $cart_update->update();
//
//        }
//        $carts = $user->carts->where('status', 'cart');
//        return view('web.cart', compact('carts'));
//
//
//    }

    public function add_to_cart(Request $request)
    {
        $this->validate($request, [
            'amount' => 'required'
        ]);
        $product = Product::find($request['product_id']);

//        $serial = request()->ip();
        $user = Auth::user();
//        if ($client) {
//            $user = $client;
//        } else {
//            $user = User::create(['serial_number' => $serial]);
//        }
        if ($request['size']) {
            $size = Size::find($request['size']);
            Cart::add($product->id, $product->name[\App::getLocale()], 1, $size->price);
            $cart = \App\Models\Cart::create([
                'product_id' => $request['product_id'],
                'size_id' => $request['size_id'],
                'price' => $request['amount'] * $size->price,
                'status' => 'cart',
                'amount' => $request['amount'],
//                'notes' => $request['notes'],
                'user_id' => $user->id,

            ]);
        } else {
            Cart::add($product->id, $product->name[\App::getLocale()], 1, $product->price);
            $cart = \App\Models\Cart::create([
                'product_id' => $request['product_id'],
                'size_id' => $request['size_id'],
                'amount' => $request['amount'],
                'price' => $request['amount'] * $product->price,
                'status' => 'cart',
//                'notes' => $request['notes'],
                'user_id' => $user->id,
            ]);
        }

        if ($request->properties) {
            foreach ($request->properties as $property) {
                $cart_properties = new CartProperties();
                $cart_properties->cart_id = $cart->id;
                $cart_properties->property_id = $property;
                $cart_properties->save();
                $price = Properties::find($property);
                if ($price) {
                    $data[] = (integer)$price->price;
                }
            }
            $price_pro = array_sum($data) * $request['amount'];
            $all_price = $price_pro + $cart->price;
            $cart_update = \App\Models\Cart::find($cart->id);
            $cart_update->price = $all_price;
            $cart_update->update();

        }
        $carts = $user->carts->where('status', 'cart');
        return view('web.cart', compact('carts'));


    }

    public function change_amount(Request $request)
    {
        $size = Size::find($request->id);
        $cart_price = $size->price;
        $data = ['status' => 1, 'msg' => 'successfully loading', 'data' => $request->id, 'cart_price' => $cart_price];

        return response($data);
    }

    public function make_order()
    {
//        $serial = request()->ip();
        $client = Auth::user();
        return view('web.order_information', compact('client'));
    }

    public function order_information(Request $request)
    {
        $this->validate($request, [
            'address' => 'required',
            'city_id' => 'required',
            'user_email' => 'required|email:rfc,dns'
        ]);
//        $serial = request()->ip();
//        $client = User::where('serial_number', $serial)->first();
        $client = Auth::user();
        if ($client) {
            $order = new Order();
            $order->user_id = $client->id;
            $order->city_id = $request['city_id'];
            $order->status = 'pending';
            $order->mobile = $client->phone;
            $order->address = $request['address'];
            $order->client_name = $client->name;
            $order->user_email = $request['user_email'];
            $order->price = $client->carts->where('status', 'cart')->sum('price');

            $order->save();

            foreach ($client->carts->where('status', 'cart') as $cart) {
                OrderCart::create([
                    'order_id' => $order->id,
                    'cart_id' => $cart->id
                ]);
                $cart->status = 'order';
                $cart->update();
            }
        } else {
            $client = Auth::user();
            $order = new Order();
            $order->user_id = $client->id;
            $order->city_id = $request['city_id'];
            $order->status = 'pending';
            $order->mobile = $request['mobile'];
            $order->address = $request['address'];
            $order->client_name = $request['name'];
            $order->user_email = $request['user_email'];
            $order->price = $client->carts->where('status', 'cart')->sum('price');

            $order->save();
            foreach ($client->carts->where('status', 'cart') as $cart) {
                OrderCart::create([
                    'order_id' => $order->id,
                    'cart_id' => $cart->id
                ]);
                $cart->status = 'order';
                $cart->update();

            }
        }
        return redirect()->route('chose_payment_method', $order->id);
    }

    public function payment_methods($id)
    {
        $order = Order::find($id);
        $payments = Payment::whereStatus('true')->get();
        return view('web.payments', compact('order', 'payments'));
    }

    public function post_payment($id, Request $request)
    {
        $order = Order::find($id);

        if ($request['lat'] == null || $request['lng'] == null) {
            return redirect()->back()->with('error', 'يرجي إختيار المكان علي الخريطة');
        };
        $this->validate($request, [
            'payment' => 'required',
        ]);

        $setting = Setting::first();
        $point1_lat = $setting->location['lat'];
        $point1_long = $setting->location['lng'];
        $point2_lat = $request['lat'];
        $point2_long = $request['lng'];

        $degrees = rad2deg(acos((sin(deg2rad($point1_lat)) * sin(deg2rad($point2_lat))) + (cos(deg2rad($point1_lat)) * cos(deg2rad($point2_lat)) * cos(deg2rad($point1_long - $point2_long)))));
        $distance = $degrees * 111.13384;

        if ($distance > 30) {
            return redirect()->back()->with('error', 'عذرا المسافه تتعدي 30 كيلو متر');
        }


        $order->payment_id = $request['payment'];
        $order->notes = $request['notes'];
        $order->lat = $request['lat'];
        $order->lng = $request['lng'];
        $order->update();

        if ($request['payment'] == 1) {

            return redirect()->route('banks', $order->id);
        } elseif ($request['payment'] == 8 || $request['payment'] == 9) {

            $amount = intval($order->price);


            return \LaravelPayfort\Facades\Payfort::redirection()->displayRedirectionPage([
                'command' => 'PURCHASE',       # AUTHORIZATION/PURCHASE according to your operation.
                'merchant_reference' => $order->id,   # You reference id for this operation (Order id for example).
                'amount' => $amount,               # The operation amount.
                'currency' => "SAR",             # Optional if you need to use another currenct than set in config.
                'customer_email' => $order->user_email,         # Customer email.
                'customer_name' => $order->client_name,
            ]);

        } else {


            foreach ($order->order_carts->where('status', 'order') as $cart) {
                $cart_product = Product::find($cart->product_id);
                $cart_product->amount -= $cart->amount;
                $cart_product->update();
            }

            return redirect()->route('home')->with('success', ' رقم الطلب الخاص بكم ' . $order->id);
        }
    }

    public function test_payment(Request $request)
    {
//        if($request['status'] == '00'){
//            $order = Order::find($request['merchant_reference']);
//            Order::destroy($order->id);
//           // return redirect()->route('home')->with('success','هناك خطأ:'.$request['response_message']);
//        }else{
//            $order = Order::find($request['merchant_reference']);
//          return redirect()->route('home')->with('success', ' رقم الطلب الخاص بكم ' . $order->id);
//        }


        if ($request['status'] == '14') {
            $order = Order::find($request['merchant_reference']);


            foreach ($order->order_carts->where('status', 'order') as $cart) {
                $cart_product = Product::find($cart->product_id);
                $cart_product->amount -= $cart->amount;
                $cart_product->update();
            }


            return redirect()->route('home')->with('success', ' رقم الطلب الخاص بكم ' . $order->id);
        } else {

            Order::destroy($request['merchant_reference']);
            return redirect()->route('home')->with('error', 'هناك خطأ:' . $request['response_message']);
        }

    }

    public function banks($id)
    {
        $order = Order::find($id);
        $banks = Bank::all();
        return view('web.banks', compact('order', 'banks'));
    }

    public function chose_bank($id, Request $request)
    {
        $order = Order::find($id);
        $this->validate($request, [
            'image' => 'required|image',
            'bank' => 'required'
        ]);
        $order->bank_id = $request['bank'];
        if ($request['image']) {
            $file_name = md5($request['image']->getClientOriginalName()) . '.' . $request['image']->getClientOriginalExtension();
            $request['image']->move(public_path('images/orders'), $file_name);
        } else {
            $file_name = null;
        }
        $order->image = $file_name;
        $order->save();
        return redirect()->route('home');
    }

    public function delegate_login()
    {
        return view('web.driver_login');
    }

    public function post_delegate_login(Request $request)
    {
        $this->validate($request, [
            'mobile' => 'required',
            'password' => 'required'
        ]);
        if (Auth::attempt(['phone' => $request['mobile'], 'password' => $request['password']])) {
            $orders = Auth::user()->drive_orders->where('status', 'pending');
            return redirect()->route('driver_orders', compact('orders'));
        } else {

            return redirect()->back();
        }
    }

    public function driver_orders()
    {
        $orders = Auth::user()->drive_orders->where('status', 'pending');
        return view('web.driver_orders', compact('orders'));
    }

    public function order_done($id)
    {
        Order::where('id', $id)->update(
            ['status' => 'finish']
        );
        return redirect()->back();
    }

    public function order_details($id)
    {
        $order = Order::find($id);
        return view('web.order_details', compact('order'));
    }

    public function rating(Request $request)
    {
        $this->validate($request, [
            'order_id' => 'required',
            'comment' => 'required',
            'rating' => 'required'
        ]);
        $order = Order::find($request['order_id']);
        if ($order) {
            Rate::create([
                'order_id' => $request['order_id'],
                'comment' => $request['comment'],
                'rate' => $request['rating']
            ]);
            return redirect()->back()->with('success', 'تم التقييم ! ');
        } else {
            return redirect()->back()->with('error', 'برجاء ادخال رقم طلب صحيح ! ');

        }
    }

    public function send_contact(Request $request)
    {
        $this->validate($request,
            [
                'title' => 'required',
                'phone' => 'required',
                'message' => 'required'
            ]);
        Contact::create([
            'title' => $request['title'],
            'phone' => $request['phone'],
            'message' => $request['message']
        ]);
        return redirect()->back()->with('success', 'تم ارسال الرسالة ');
    }

    public function delete_cart($id)
    {
        \App\Models\Cart::destroy($id);
        return redirect()->route('home')->with('success', 'تم الحذف');
    }

    public function user_cart()
    {
//        $serial = request()->ip();
//        $user = User::where('serial_number', $serial)->first();
        $user = Auth::user();
        $carts = $user->carts->where('status', 'cart');
        if ($carts) {
            return view('web.cart', compact('carts'));
        } else {
            return back()->with('success', 'لا يوجد مشتروات');
        }
    }

    public function testing()
    {
        return view('test');
    }


    public function get_login()
    {
        if (Auth::check()) {
            return redirect()->route('home');
        }
        return view('web.login');
    }

    public function user_login(Request $request)
    {
        $this->validate($request, [
            'phone' => 'required',
            'password' => 'required'
        ]);
        $login = \Auth::attempt(['phone' => $request['phone'], 'password' => $request['password'], 'status' => 'active', 'type' => 'client']);
        if ($login) {
            return redirect()->route('home');
        } else {
            return redirect()->back()->with('error', \Lang::get('data.error_login_credentials'));
        }
    }


    public function get_register()
    {
        if (Auth::check()) {
            return redirect()->route('home');
        }
        return view('web.register');
    }


    public function store_register(Request $request)
    {
        $this->validate($request, [
            'phone' => 'required|unique:users,phone',
            'name' => 'required|unique:users,name',
            'password' => 'required|min:6'
        ]);


        $user = User::create([
            'name' => $request['name'],
            'phone' => $request['phone'],
            'password' => bcrypt($request['password']),
            'type' => 'client',
            'status' => 'active',
        ]);
        \Auth::login($user);
        return redirect()->back()->with('message', \Lang::get('data.welcome'));


    }

    public function my_account()
    {
        $row = Auth::user();
        return view('web.my_account', compact('row'));
    }

    public function update_user(Request $request, $id)
    {
        $this->validate($request, [
            'phone' => 'required|unique:users,phone,' . $id,
            'name' => 'required|unique:users,name,' . $id
        ]);

        $user = User::find($id);
        $user->name = $request['name'];
        $user->phone = $request['phone'];
        if ($request['password']) {
            $user->password = bcrypt($request['password']);
        }
        $user->update();
        return redirect()->back()->with('success', 'تم بنجاح');
    }

    public function get_send_code()
    {
        return view('web.forget_password');
    }


    public function send_code(Request $request)
    {
        $user = User::whereEmail($request->email)->first();
        if ($user) {
            $code = rand(1111, 9999);
            $user->code = $code;
            $user->update();

            \Mail::to($user->email)->send(new ForgetPassword($code));
            \Session::put('email', $user->email);

            return redirect()->route('confirm_code')->with('success', 'تم ارسال كود التفعيل');
        }
        return redirect()->back()->with('error', 'لا يوجد مستخدم بهذا البريد الالكترونى');

    }

    public function confirm_code(Request $request)
    {
        $email = \Session::get('email');
        if ($email) {
            return view('web.confirm_code');
        }
        return view('web.index');

    }

    public function store_confirm_code(Request $request)
    {
        $this->validate($request, [
            'code' => 'required'
        ]);
        $email = \Session::get('email');

        $user = User::where('email', $email)->first();
        if ($user->code == $request->code) {
            \Session::put('user_id', $user->id);
            return view('web.change_password');
        }
        return redirect()->back()->with('error', 'الرمز غير صحيح');
    }


    public function store_change_password(Request $request)
    {
        $this->validate($request, [
            'password' => 'required|min:6'
        ]);
        $user_id = \Session::get('user_id');
        $user = User::find($user_id);
        $user->password = bcrypt($request['password']);
        $user->update();
        \Auth::login($user);
        \Session::forget('user_id');
        \Session::forget('email');

        return redirect()->route('home');

    }

    public function logout()
    {
        \Auth::logout();
        return redirect()->route('home')->with('success', 'تم تسجيل الخروج بنجاح');
    }

}
