<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use App\Models\Properties;
use App\Models\Category;
class Product extends Model
{

    protected $table = 'products';
    public $timestamps = true;

    use SoftDeletes;

    protected $dates = ['deleted_at'];
    protected $fillable = array('name', 'category_id', 'price', 'notes', 'type', 'status', 'size_name', 'barcode', 'amount_id', 'amount');
    protected $casts = ['name' => 'json', 'notes' => 'json', 'size_name' => 'json'];
    protected $appends = ['check_properties'];

    protected function getCheckPropertiesAttribute()
    {
$lang  = request()->header('Accept-Language','ar');
        $parents = [];
        foreach ($this->properties as $property) {
            $parents[] = $property->parent_id;
        }

        $count = 0;
        $unique_parents = array_unique($parents);
        $categories = Properties::whereIn('id', $unique_parents)->get();

        $data['properties'] = [];
        foreach ($categories as $category) {
            if (!$this->properties->contains($category)) {
                $count++;
                $arr['id'] = $category->id;
                $arr['name'] = $category->name[$lang];
                $arr['price'] = $category->price ? (string)$category->price : '';
                $arr['sub_properties'] = [];
                foreach ($category->childs as $child) {


                    if ($this->properties->contains($child)) {

                        $ch['id'] = $child->id;
                        $ch['name'] = $child->name[$lang];
                        $ch['price'] = $child->price ? (string)$child->price : '';

                        $ch['child_properties'] = [];
                        foreach ($child->childs as $child_property) {
                            if ($this->properties->contains($child_property)) {
                                $ch_p['id'] = $child_property->id;
                                $ch_p['name'] = $child_property->name[$lang];
                                $ch_p['price'] = $child_property->price ? (string)$child_property->price : '';
                                $ch['child_properties'][] = $ch_p;

                            }
                        }
                        $arr['sub_properties'][] = $ch;
                    }

                }
                $data['properties'][] = $arr;
            }

        }
        return $count;
    }

    public function category()
    {
        return $this->belongsTo('App\Models\Category', 'category_id');
    }

    public function additionals()
    {
        return $this->hasMany('App\Models\Additional', 'product_id', 'id');
    }

    public function carts()
    {
        return $this->hasMany('App\Models\Cart', 'cart_id', 'id');
    }

    public function images()
    {
        return $this->hasMany('App\Models\Image', 'product_id', 'id');
    }

    public function sizes()
    {
        return $this->hasMany('App\Models\Size', 'product_id', 'id');
    }

    public function pieces()
    {
        return $this->hasMany('App\Models\Piece', 'product_id', 'id');
    }

    public function amountType()
    {
        return $this->belongsTo(AmountType::class, 'amount_id');
    }

    public function activate()
    {
        $var = route('active_product', ['id' => $this->id]);
        $token = csrf_token();
        if ($this->status == 'active') {
            return "<form style='margin-top: 20px' method='POST' action='$var' class='form-horizontal'>
                <input type='hidden' name='_token' value='$token'>
                <button type='submit' class='btn btn-danger btn-rounded waves-effect waves-light'>
                <span class='btn-label'><i class='fa fa-times'></i></span>
                الغاء التفعيل</button>
                
            </form>";
        }
        return "<form style='margin-top: 20px' method='POST' action='$var' class='form'>
                <input type='hidden' name='_token' value='$token'>
                <button type='submit' class='btn btn-success btn-rounded waves-effect waves-light'>
                <span class='btn-label'><i class='fa fa-check'></i></span>
                تفعيل</button>
                
            </form>";
    }

    public function properties()
    {
        return $this->belongsToMany(Properties::class, 'product_properties', 'product_id', 'property_id');
    }
}