<?php

namespace App;

use App\Models\Cart;
use App\Models\Message;
use App\Models\Notification;
use App\Models\Rate;
use App\Models\Report;
use App\Models\Transfer;
use Carbon\Carbon;
use App\Models\Order;
use Illuminate\Notifications\Notifiable;
use Illuminate\Foundation\Auth\User as Authenticatable;
use App\Models\Service;
use Tymon\JWTAuth\Contracts\JWTSubject;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;


class User extends Authenticatable implements JWTSubject
{
    use Notifiable;

    //  use HasRoles;

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    public function scopeActive($query)
    {
        $query->where('status', 'active');
    }

//    public function setImageAttribute($image)
//    {
//        $file = request()->file('image');
//        $destinationPath = 'images/user/';
//        $filename = $file->getClientOriginalName();
//        $file->move($destinationPath, $filename);
//        $this->attributes['image'] = $filename;
//    }

//    public function getImageAttribute()
//    {
//        if ($this->attributes['image']) {
//            return asset('images/user/') . '/' . $this->attributes['image'];
//        }
//        return asset('images/user/admin.png');
//    }

//    public function setPasswordAttribute($password)
//    {
//        if (isset($password)) {
//            $this->attributes['password'] = bcrypt($password);
//        }
//    }

    public function getDateOfRegisterAttribute($value)
    {
        return Carbon::parse($value)->format('d/m/Y');
    }

    public function activate()
    {
        $var = route('active_user', ['id' => $this->id]);
        $token = csrf_token();
        if ($this->status == 'active') {
            return "<form style='margin-top: 20px' method='POST' action='$var' class='form-horizontal'>
                <input type='hidden' name='_token' value='$token'>
                <button type='submit' class='btn btn-danger btn-rounded waves-effect waves-light'>
                <span class='btn-label'><i class='fa fa-times'></i></span>
                الغاء التفعيل</button>

            </form>";
        }
        return "<form style='margin-top: 20px' method='POST' action='$var' class='form'>
                <input type='hidden' name='_token' value='$token'>
                <button type='submit' class='btn btn-success btn-rounded waves-effect waves-light'>
                <span class='btn-label'><i class='fa fa-check'></i></span>
                تفعيل</button>

            </form>";
    }





    protected $table = 'users';
    public $timestamps = true;

    use SoftDeletes;

    protected $dates = ['deleted_at'];
    protected $fillable = array('name', 'password', 'city_id', 'address', 'phone', 'type', 'serial_number','status','device_token','lat','lng','device_type','email');



    /**
     * The attributes that should be hidden for arrays.
     *
     * @var array
     */
    protected $hidden = ['password', 'remember_token',];
    protected $casts = [
        'device_token' => 'json',
    ];
    // relations
    public function getJWTIdentifier()
    {
        return $this->getKey();
    }

    public function getJWTCustomClaims()
    {
        return [];
    }
    // relations
    public function orders(){
        return $this->hasMany('App\Models\Order','user_id','id');
    }
    public function city(){
        return $this->belongsTo('App\Models\City','city_id','id');
    }
    public function drive_orders(){
        return $this->hasMany('App\Models\Order','driver_id','id');
    }
    public function carts(){
        return $this->hasMany('App\Models\Cart','user_id','id');
    }
}
